@@ -35,8 +35,9 @@ Marks
    Map
 """
 from ipywidgets import Widget, DOMWidget, CallbackDispatcher, Color, widget_serialization
-from traitlets import (Int, Unicode, List, Enum, Dict, Bool, Float, TraitError,
-                       Instance, Tuple)
+from traitlets import (
+        Int, Unicode, List, Enum, Dict, Bool, Float, TraitError, Instance, Tuple
+    )
 
 from .scales import Scale, OrdinalScale
 from .traits import NdArray, Date
@@ -160,10 +161,35 @@ class Mark(Widget):
     _view_module = Unicode('bqplot').tag(sync=True)
     _ipython_display_ = None
 
+    def _get_dimension_scales(self, dimension, preserve_domain=False):
+        """
+        Return the list of scales corresponding to a given dimension.
+
+        The preserve_domain optional argument specifies whether one should
+        filter out the scales for which preserve_domain is set to True.
+        """
+        if preserve_domain:
+            return [
+                self.scales[k] for k in self.scales if (
+                    k in self.scales_metadata
+                    and self.scales_metadata[k].get('dimension') == dimension
+                    and not self.preserve_domain.get(k)
+                )
+            ]
+        else:
+            return [
+                self.scales[k] for k in self.scales if (
+                    k in self.scales_metadata
+                    and self.scales_metadata[k].get('dimension') == dimension
+                )
+            ]
+
     def _scales_validate(self, scales, scales_trait):
-        """validates the dictionary of scales based on the mark's scaled
-        attributes metadata. First checks for missing scale and then for
-        'rtype' compatibility."""
+        """
+        Validates the `scales` based on the mark's scaled attributes metadata.
+
+        First checks for missing scale and then for 'rtype' compatibility.
+        """
         # Validate scales' 'rtype' versus data attribute 'rtype' decoration
         # At this stage it is already validated that all values in self.scales
         # are instances of Scale.
@@ -805,8 +831,16 @@ class Label(Mark):
     enable_move: Bool
         Enable the label to be moved by dragging
     """
-    x = Float().tag(sync=True) | Date().tag(sync=True) | Unicode().tag(sync=True)  # TODO: Why could it be Unicode
-    y = Float().tag(sync=True) | Date().tag(sync=True) | Unicode().tag(sync=True)
+    x = NdArray().tag(sync=True, min_dim=1, max_dim=1, atype='bqplot.Axis')
+    y = NdArray().tag(sync=True, min_dim=1, max_dim=1, atype='bqplot.Axis')
+    color = NdArray(None, allow_none=True).tag(sync=True, scaled=True,
+                rtype='Color', atype='bqplot.ColorAxis', min_dim=1, max_dim=1)
+    size = NdArray(None, allow_none=True).tag(sync=True, scaled=True,
+                   rtype='Number', min_dim=1, max_dim=1)
+    rotation = NdArray(None, allow_none=True).tag(sync=True, scaled=True,
+                       rtype='Number', min_dim=1, max_dim=1)
+    opacity = NdArray(None, allow_none=True).tag(sync=True, scaled=True,
+                      rtype='Number', min_dim=1, max_dim=1)
     x_offset = Int().tag(sync=True)
     y_offset = Int().tag(sync=True)
     scales_metadata = Dict({
@@ -814,14 +848,47 @@ class Label(Mark):
         'y': { 'orientation': 'vertical', 'dimension': 'y' },
         'color': { 'dimension': 'color' }
     }).tag(sync=True)
-    color = Color(None, allow_none=True).tag(sync=True)
+    colors = List(trait=Color(default_value=None, allow_none=True), default_value=CATEGORY10).tag(sync=True, display_name='Colors')
+    default_opacities = List(trait=Float(1.0, min=0, max=1, allow_none=True)).tag(sync=True, display_name='Opacities')
     rotate_angle = Float().tag(sync=True)
-    text = Unicode().tag(sync=True)
-    font_size = Unicode(default_value='14px').tag(sync=True)
+    text = NdArray().tag(sync=True)
+    font_size = Float(16.).tag(sync=True)
+    font_unit = Enum(['px', 'em', 'pt', '%'], default_value='px').tag(sync=True)
     font_weight = Enum(['bold', 'normal', 'bolder'], default_value='bold').tag(sync=True)
     align = Enum(['start', 'middle', 'end'], default_value='start').tag(sync=True)
+
     enable_move = Bool().tag(sync=True)
-    
+    restrict_x = Bool().tag(sync=True)
+    restrict_y = Bool().tag(sync=True)
+    update_on_move = Bool().tag(sync=True)
+
+    def __init__(self, **kwargs):
+        self._drag_start_handlers = CallbackDispatcher()
+        self._drag_handlers = CallbackDispatcher()
+        self._drag_end_handlers = CallbackDispatcher()
+        super(Label, self).__init__(**kwargs)
+
+    def on_drag_start(self, callback, remove=False):
+        self._drag_start_handlers.register_callback(callback, remove=remove)
+
+    def on_drag(self, callback, remove=False):
+        self._drag_handlers.register_callback(callback, remove=remove)
+
+    def on_drag_end(self, callback, remove=False):
+        self._drag_end_handlers.register_callback(callback, remove=remove)
+
+    def _handle_custom_msgs(self, _, content, buffers=None):
+        event = content.get('event', '')
+
+        if event == 'drag_start':
+            self._drag_start_handlers(self, content)
+        elif event == 'drag':
+            self._drag_handlers(self, content)
+        elif event == 'drag_end':
+            self._drag_end_handlers(self, content)
+
+        super(Label, self)._handle_custom_msgs(self, content)
+
     _view_name = Unicode('Label').tag(sync=True)
     _model_name = Unicode('LabelModel').tag(sync=True)
 
@@ -984,7 +1051,7 @@ class Pie(Mark):
     display_labels = Bool(True).tag(sync=True)
     label_color = Color(None, allow_none=True).tag(sync=True)
     font_size = Unicode(default_value='10px').tag(sync=True)
-    font_weight = Enum(['bold', 'normal', 'bolder'], 
+    font_weight = Enum(['bold', 'normal', 'bolder'],
                        default_value='normal').tag(sync=True)
 
     _view_name = Unicode('Pie').tag(sync=True)
@@ -1143,7 +1210,7 @@ class GridHeatMap(Mark):
 
     row_align = Enum(['start', 'end'], default_value='start').tag(sync=True)
     column_align = Enum(['start', 'end'], default_value='start').tag(sync=True)
-
+    null_color = Color('black', allow_none=True).tag(sync=True)
     stroke = Color('black', allow_none=True).tag(sync=True)
     opacity = Float(1.0, min=0.2, max=1).tag(sync=True, display_name='Opacity')
     anchor_style = Dict({'fill': 'white', 'stroke': 'blue'}).tag(sync=True)
