@@ -88,12 +88,12 @@ class Date(TraitType):
         if value is None:
             return value
         else:
-            return value.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
+            return value.strftime('%Y-%m-%dT%H:%M:%S.%f')
 
     @staticmethod
     def from_json(value, obj=None):
         if value:
-            return dt.datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%fZ')
+            return dt.datetime.strptime(value, '%Y-%m-%dT%H:%M:%S.%f')
         else:
             return value
 
@@ -135,13 +135,33 @@ class NdArray(CInstance):
             return {'values': a, 'type': None}
 
     def _set_value(self, value):
-        if value is None or len(value) == 0:
-            return np.asarray(value)
-        if(isinstance(value, np.ndarray) and np.issubdtype(value.dtype, np.datetime64)):
+        if isinstance(value, np.ndarray):
             return value
+        elif value is None or len(value) == 0:
+            return np.asarray(value)
         else:
             return np.asarray(value)
 
+    def set(self, obj, value):
+        #TODO: We shouldnt have to overload the set because
+        # numpy doesn't support == comparions
+        new_value = self._validate(obj, value)
+        try:
+            old_value = obj._trait_values[self.name]
+        except KeyError:
+            old_value = self.default_value
+
+        obj._trait_values[self.name] = new_value
+        try:
+            silent = np.array_equal(old_value, new_value)
+        except:
+            # if there is an error in comparing, default to notify
+            silent = False
+        if silent is not True:
+            # we explicitly compare silent to True just in case the equality
+            # comparison above returns something other than True/False
+            obj._notify_trait(self.name, old_value, new_value)
+
     def validate(self, obj, value):
         # If it is an object, I have to check if it can be cast into a date
         if (not isinstance(value, self.klass) or value.dtype == 'object'):
@@ -153,7 +173,9 @@ class NdArray(CInstance):
         if (dim > 1) and (1 in shape):
             value = np.squeeze(value) if (self.squeeze) else value
             dim = len(np.shape(value))
-        if dim > max_dim or dim < min_dim:
+        if self.allow_none and dim == 0:
+            return value
+        if (dim > max_dim or dim < min_dim):
             raise TraitError("Dimension mismatch")
         return value
 
@@ -165,11 +187,9 @@ class NdArray(CInstance):
             return np.asarray(value['values'], dtype=array_dtype)
 
     def __init__(self, *args, **kwargs):
-        kwargs.setdefault('from_json', NdArray._from_json)
-        kwargs.setdefault('to_json', NdArray._to_json)
-        kwargs.setdefault('args', (0,))
         self.squeeze = kwargs.pop('squeeze', True)
         super(NdArray, self).__init__(*args, **kwargs)
+        self.tag(to_json=NdArray._to_json, from_json=NdArray._from_json)
 
     _cast = _set_value
 
@@ -227,15 +247,13 @@ class PandasDataFrame(Instance):
     info_text = 'a pandas DataFrame'
 
     def __init__(self, *args, **kwargs):
-        kwargs.setdefault('from_json', self._from_json)
-        kwargs.setdefault('to_json', self._to_json)
-        kwargs.setdefault('args', ())
         super(PandasDataFrame, self).__init__(*args, **kwargs)
+        self.tag(to_json=self._to_json, from_json=self._from_json)
 
     def _from_json(self, value, obj=None):
-        if value is not None:
+        if value is not None and value != []:
             df = pd.DataFrame(value)
-            df = df.set_index('index')
+            df = df.set_index(self.index_name)
             df.index.name = None
         else:
             df = pd.DataFrame()
@@ -243,10 +261,36 @@ class PandasDataFrame(Instance):
 
     def _to_json(self, df, obj=None):
         if df is not None:
+            # Name of the index column
+            self.index_name = df.index.name
+            if self.index_name is None:
+                # If the index is not named, reset_index() names the column 'index'
+                self.index_name = 'index'
             return df.reset_index().to_dict(orient='records')
         else:
             return []
 
+    def set(self, obj, value):
+        new_value = self._validate(obj, value)
+        try:
+            old_value = obj._trait_values[self.name]
+        except KeyError:
+            old_value = self.default_value
+
+        obj._trait_values[self.name] = new_value
+        try:
+            if new_value is not None:
+                silent = new_value.equals(old_value)
+            else:
+                silent = bool(new_value == old_value)
+        except:
+            # if there is an error in comparing, default to notify
+            silent = False
+        if silent is not True:
+            # we explicitly compare silent to True just in case the equality
+            # comparison above returns something other than True/False
+            obj._notify_trait(self.name, old_value, new_value)
+
     def validate(self, obj, value):
         value = super(PandasDataFrame, self).validate(obj, value)
         if self.get_metadata('lexsort'):
@@ -254,8 +298,6 @@ class PandasDataFrame(Instance):
                 value = value.sortlevel(0, axis=1)
         return value
 
-    _cast = _from_json
-
 
 class PandasSeries(Instance):
 
@@ -270,10 +312,8 @@ class PandasSeries(Instance):
     info_text = 'a pandas series'
 
     def __init__(self, *args, **kwargs):
-        kwargs.setdefault('from_json', self._from_json)
-        kwargs.setdefault('to_json', self._to_json)
-        kwargs.setdefault('args', ())
         super(PandasSeries, self).__init__(*args, **kwargs)
+        self.tag(to_json=self._to_json, from_json=self._from_json)
 
     def _from_json(self, value, obj=None):
         return pd.Series(value)
